@@ -1,6 +1,6 @@
 """core.py: library code for the pandoc-xnos filter suite."""
 
-# Copyright 2015-2019 Thomas J. Duck.
+# Copyright 2015-2020 Thomas J. Duck.
 # All rights reserved.
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-__version__ = '2.0.1'
+__version__ = '2.4.2'
 
 
 import os
@@ -27,6 +27,8 @@ import re
 import textwrap
 import functools
 import copy
+import collections
+import inspect
 
 import psutil
 
@@ -63,8 +65,9 @@ else:
     # See: https://docs.python.org/2/howto/unicode.html.
     STDIN = sys.stdin
     STDOUT = sys.stdout
-    STDERR = sys.stdout
+    STDERR = sys.stderr
 
+# Global state-tracking variables
 # pylint: disable=invalid-name
 _cleveref_flag = None  # Flags that the cleveref package is needed
 _sec = None            # Used to track section numbers
@@ -90,44 +93,43 @@ def _repeat(func):
         return ret
     return wrapper
 
+# Compatibility layer for deprecated 2.0.1 api.  This applies to two functions:
+# process_refs_factory() and attach_attrs_factory().
+def _compat(f):
+    """Compatibility layer."""
+    @functools.wraps(f)
+    def wrapper(*args, **kwargs):
+        """Wrapper."""
+        if 'patt' in kwargs:
+            kwargs['regex'] = kwargs.pop('patt')
+        if args and isinstance(args[0], STRTYPES):
+            try:  # This may be a call to the deprecated 2.0.1 api
+                return f(*args[1:], **kwargs)
+            except TypeError:  # Usually a signature mismatch; use new api
+                pass
+        kwargs.pop('name', None)
+        return f(*args, **kwargs)
+    return wrapper
+
 
 #=============================================================================
 # Utility functions
 
 # init() ---------------------------------------------------------------------
 
-_PANDOCVERSION = None  # A string giving the pandoc version
+_PANDOCVERSION = None    # A string giving the pandoc version
+_FILTERNAME = None       # The name of the calling filter
+_WARNINGLEVEL = 2  # 0 for no warnings; 1 for critical warnings; 2 for all
 
-# pylint: disable=too-many-branches
-def init(pandocversion=None, doc=None):
-    """Initializes library.  This must be called.
+def _get_pandoc_version(pandocversion, doc):
+    """Determines, checks, and returns the pandoc version."""
 
-    This function:
+    # This requires some care because we can't be sure that a call to `pandoc`
+    # will work.  It could be `pandoc-1.17.0.2` or some other name.  Try
+    # checking the PANDOC_VERSION environment variable, and then the parent
+    # process.  Only make a call to `pandoc` as a last resort.
 
-      1) Sets (or resets) global variables.
-
-      2) Sets or determines the pandoc version, which is needed for
-         multi-version support.  See: https://github.com/jgm/pandoc/issues/2640
-
-    Returns the pandoc version.
-    """
-
-    # This requires some care because we can't be sure that a call to 'pandoc'
-    # will work.  It could be 'pandoc-1.17.0.2' or some other name.  Try
-    # checking the parent process first, and only make a call to 'pandoc' as
-    # a last resort.
-
-    # pylint: disable=global-statement
-    global _PANDOCVERSION
-    global _cleveref_flag
-    global _sec
-
-    # Set (or reset) globals
-    _cleveref_flag = None  # Flags that the cleveref package is needed
-    _sec = 0            # Used to track section numbers
-
-    # Get the pandoc version
-    if pandocversion:  # It was provided
+    if pandocversion:  # It was provided; only need to check it
         pass
 
     elif 'PANDOC_VERSION' in os.environ:  # Available for pandoc >= 1.19.1
@@ -135,8 +137,8 @@ def init(pandocversion=None, doc=None):
 
     elif doc is not None and 'pandoc-api-version' in doc:
         # This could be pandoc 1.18 or 1.19; there is no way to distinguish
-        # them (but there isn't a use case in pandoc-fignos and friends where
-        # it matters).
+        # them.  Fortunately, but there isn't a use case in pandoc-fignos and
+        # friends where it matters.
         pandocversion = '1.18'
 
     else:
@@ -147,6 +149,9 @@ def init(pandocversion=None, doc=None):
                 command = psutil.Process(os.getpid()).parent().parent().exe()
             else:
                 command = psutil.Process(os.getpid()).parent().exe()
+            # Expect the executable to begin with the name `pandoc`;
+            # e.g., `pandoc-2.7.3`.  If it does not, then the parent process
+            # could be another program altogether (e.g., `panzer`).
             if not os.path.basename(command).startswith('pandoc'):
                 raise RuntimeError('pandoc not found')
         except:  # pylint: disable=bare-except
@@ -162,12 +167,11 @@ def init(pandocversion=None, doc=None):
         except: # pylint: disable=bare-except
             pass
 
-    # Test `pandocversion` and if it is OK then store and return it
+    # Test `pandocversion` and if it is OK then return it
     pattern = re.compile(r'^[1-2]\.[0-9]+(?:\.[0-9]+)?(?:\.[0-9]+)?$')
     if pandocversion is not None:
         if pattern.match(pandocversion):
-            _PANDOCVERSION = pandocversion
-            return _PANDOCVERSION
+            return pandocversion
         msg = 'Cannot understand pandocversion=%s'%pandocversion
         raise RuntimeError(msg)
 
@@ -178,10 +182,63 @@ def init(pandocversion=None, doc=None):
     raise RuntimeError(msg)
 
 
+# pylint: disable=too-many-branches
+def init(pandocversion=None, doc=None):
+    """Initializes library.  This must be called before a filter accesses
+    other functions in this library.
+
+    This function:
+
+      1) Sets (or resets) global variables.
+      2) Sets or determines the pandoc version.
+      3) Inspects and saves the calling module's name
+
+    Returns the pandoc version.
+
+    Parameters:
+
+      pandocversion - A string representing the pandoc-version; if this is
+                      None then init() will attempt to determine the version
+                      through other means
+      doc - the pandoc document AST dict
+    """
+
+    # pylint: disable=global-statement
+    global _PANDOCVERSION
+    global _FILTERNAME
+    global _cleveref_flag
+    global _sec
+
+    # Set (or reset) globals
+    _cleveref_flag = None  # Flags that the cleveref package is needed
+    _sec = 0            # Used to track section numbers
+
+    # Save the calling module's name; see https://stackoverflow.com/a/1095621
+    frame = inspect.stack()[1][0]
+    _FILTERNAME = inspect.getmodule(frame).__name__.replace('_', '-')
+
+    # Get and return the pandoc version
+    _PANDOCVERSION = _get_pandoc_version(pandocversion, doc)
+    return _PANDOCVERSION
+
+
+def version(v):
+    """Converts a version string into a tuple appropriate for comparisons."""
+    return tuple(int(x) for x in v.split('.'))
+
+
+# set_warning_level() --------------------------------------------------------
+def set_warning_level(n):
+    """0 for no warnings; 1 for critical warnings; 2 for all warnings"""
+    # pylint: disable=global-statement
+    global _WARNINGLEVEL
+    _WARNINGLEVEL = n
+
+
 # check_bool() ---------------------------------------------------------------
 
 def check_bool(v):
-    """Checks that metadata value is boolean.  Returns the value or
+    """Checks that metadata value `v` is boolean.  Returns the value or
     raises an exception."""
     if not isinstance(v, bool):
         msg = 'Metadata boolean values must be one of the following: ' \
@@ -198,8 +255,9 @@ def check_bool(v):
 # command line or in a document.  The get_meta() function makes no
 # distinction.
 
+# pylint: disable=too-many-return-statements
 def get_meta(meta, name):
-    """Retrieves the metadata variable 'name' from the 'meta' dict."""
+    """Retrieves the metadata variable `name` from the `meta` dict."""
     assert name in meta
     data = meta[name]
 
@@ -215,6 +273,17 @@ def get_meta(meta, name):
                 return False
         return stringify(data['c'])
     if data['t'] == 'MetaList':
+        try:  # Process MetaList of MetaMaps
+            ret = []
+            for v in data['c']:
+                assert v['t'] == 'MetaMap'
+                entry = {}
+                for key in v['c']:
+                    entry[key] = stringify(v['c'][key])
+                ret.append(entry)
+            return ret
+        except AssertionError:
+            pass
         return [stringify(v['c']) for v in data['c']]
     if data['t'] == 'MetaMap':
         ret = {}
@@ -227,8 +296,7 @@ def get_meta(meta, name):
 # elt() ----------------------------------------------------------------------
 
 def elt(eltType, numargs):  # pylint: disable=invalid-name
-    """Returns Element(*value) to create pandoc AST elements.
-
+    """Returns Element(*value) function to create pandoc AST elements.
     This should be used in place of pandocfilters.elt().  This version
     ensures that the content is stored in a list, not a tuple.
     """
@@ -243,7 +311,7 @@ def elt(eltType, numargs):  # pylint: disable=invalid-name
 Cite = elt('Cite', 2)  # pylint: disable=invalid-name
 
 def _getel(key, value):
-    """Returns an element given a key and value."""
+    """Returns an element given a `key` and `value`."""
     if key in ['HorizontalRule', 'Null']:
         return elt(key, 0)()
     if key in ['Plain', 'Para', 'BlockQuote', 'BulletList',
@@ -258,8 +326,27 @@ def _getel(key, value):
 # meta variable in post-filter processing.  This owing to a design decision
 # in pandoc.  See https://github.com/jgm/pandoc/issues/3139.
 
-def add_to_header_includes(meta, fmt, block, warninglevel, regex=None):
-    """Adds block to header-includes in metadata."""
+def add_to_header_includes(meta, fmt, block, warninglevel=None, regex=None):
+    """Adds `block` to header-includes field of `meta`.  The block is
+    encapsulated in a pandoc RawBlock.
+
+    Parameters:
+
+      meta - the document metadata
+      fmt - the format of the block (tex, html, ...)
+      block - the block of text to add to the header-includes
+      warninglevel - DEPRECATED (set global WARNINGLEVEL instead)
+      regex - a regular expression used to check existing header-includes
+              in the document metadata for overlap
+    """
+
+    # Set the global warning level (DEPRECATED)
+    # pylint: disable=global-statement
+    global _WARNINGLEVEL
+    assert(warninglevel is None or isinstance(warninglevel, int))
+    if warninglevel is not None:
+        _WARNINGLEVEL = warninglevel
+
     # If pattern is found in the meta-includes then bail out
     if regex and 'header-includes' in meta:
         pattern = re.compile(regex)
@@ -284,12 +371,12 @@ def add_to_header_includes(meta, fmt, block, warninglevel, regex=None):
             """ % str(meta['header-includes']))
         raise RuntimeError(msg)
     # Print the block to stderr at warning level 2
-    if warninglevel == 2:
+    if _WARNINGLEVEL == 2:
         if hasattr(textwrap, 'indent'):
             STDERR.write(textwrap.indent(block, '    '))
         else:
             STDERR.write('\n'.join('    ' + line for line in block.split('\n')))
-            STDERR.flush()
+        STDERR.flush()
 
 
 # cleveref_required() --------------------------------------------------------
@@ -306,16 +393,17 @@ def cleveref_required():
 # quotify() ------------------------------------------------------------------
 
 def quotify(x):
-    """Replaces Quoted elements in element list 'x' with quoted strings.
+    """Replaces Quoted elements in element list `x` with quoted strings.
 
-    Pandoc uses the Quoted element in its json when --smart is enabled.
-    Output to TeX/pdf automatically triggers --smart.
+    Pandoc uses the Quoted element when '--smart' is enabled.  Note that
+    output to TeX/pdf automatically triggers '--smart'.
 
     stringify() ignores Quoted elements.  Use quotify() first to replace
-    Quoted elements in 'x' with quoted strings.  'x' should be a deep copy so
+    Quoted elements in `x` with quoted strings.  `x` should be a deep copy so
     that the underlying document is left untouched.
 
-    Returns x."""
+    Returns `x`, modified in-place.
+    """
 
     def _quotify(key, value, fmt, meta):  # pylint: disable=unused-argument
         """Replaced Quoted elements with quoted strings."""
@@ -341,13 +429,14 @@ def quotify(x):
 # dollarfy() -----------------------------------------------------------------
 
 def dollarfy(x):
-    """Replaces Math elements in element list 'x' with a $-enclosed string.
+    """Replaces Math elements in element list `x` with a $-enclosed string.
 
     stringify() passes through TeX math.  Use dollarfy(x) first to replace
-    Math elements with math strings set in dollars.  'x' should be a deep copy
+    Math elements with math strings set in dollars.  `x` should be a deep copy
     so that the underlying document is left untouched.
 
-    Returns 'x'."""
+    Returns `x`, modified in-place.
+    """
 
     def _dollarfy(key, value, fmt, meta):  # pylint: disable=unused-argument
         """Replaces Math elements"""
@@ -361,14 +450,15 @@ def dollarfy(x):
 # extract_attrs() ------------------------------------------------------------
 
 def extract_attrs(x, n):
-    """Extracts attributes from element list 'x' beginning at index 'n'.
+    """Extracts attributes from element list `x` beginning at index `n`.
 
     The elements encapsulating the attributes (typically a series of Str and
-    Space elements) are removed from 'x'.  Items before index 'n' are left
+    Space elements) are removed from `x`.  Items before index `n` are left
     unchanged.
 
     Returns the attributes.  A ValueError is raised if attributes aren't
-    found.  An IndexError is raised if the index 'n' is out of range."""
+    found.  An IndexError is raised if the index `n` is out of range.
+    """
 
     # Check for the start of the attributes string
     if not (x[n]['t'] == 'Str' and x[n]['c'].startswith('{')):
@@ -450,7 +540,7 @@ def extract_attrs(x, n):
 
 @_repeat
 def _join_strings(x, start=0):
-    """Joins adjacent Str elements found in the element list 'x'."""
+    """Joins adjacent Str elements found in the element list `x`."""
     for i in range(start, len(x)-1):  # Process successive pairs of elements
         if x[i]['t'] == 'Str' and x[i+1]['t'] == 'Str':
             x[i]['c'] += x[i+1]['c']
@@ -460,7 +550,7 @@ def _join_strings(x, start=0):
 
 # pylint: disable=unused-argument
 def join_strings(key, value, fmt=None, meta=None):
-    """Joins adjacent Str elements in the 'value' list."""
+    """Joins adjacent Str elements found in `value`."""
     if key in ['Para', 'Plain']:
         _join_strings(value)
     elif key == 'Span':
@@ -468,24 +558,30 @@ def join_strings(key, value, fmt=None, meta=None):
     elif key == 'Image':
         _join_strings(value[-2])
     elif key == 'Table':
-        _join_strings(value[-5])
+        if version(_PANDOCVERSION) < version('2.10'):
+            _join_strings(value[-5])
+        else:
+            _join_strings(value[-5]['c'][1][0]['c'])
 
 
-# repair_reference() ---------------------------------------------------------
 
-# Reference regex.  This splits a reference into three components: the
+# repair_refs() -------------------------------------------------------------
+
+# Reference pattern.  This splits a reference into three components: the
 # prefix, label and suffix.  e.g.:
 # >>> _REF.match('xxx{+@fig:1}xxx').groups()
 # ('xxx{+', 'fig:1', '}xxx').
 _REF = re.compile(r'^((?:.*{)?[\*\+!]?)@([^:]*:[\w/-]+)(.*)')
 
 def _is_broken_ref(key1, value1, key2, value2):
-    """True if this is a broken reference; False otherwise."""
+    """True if the keys and values represent a broken reference;
+    False otherwise.
+    """
     # A link followed by a string may represent a broken reference
     if key1 != 'Link' or key2 != 'Str':
         return False
     # Assemble the parts
-    n = 0 if _PANDOCVERSION < '1.16' else 1
+    n = 0 if version(_PANDOCVERSION) < version('1.16') else 1
 
     if isinstance(value1[n][0]['c'], list):
         # Occurs when there is quoted text in an actual link.  This is not
@@ -493,12 +589,12 @@ def _is_broken_ref(key1, value1, key2, value2):
         return False
 
     s = value1[n][0]['c'] + value2
-    # Return True if this matches the reference regex
+    # Return True if this matches the reference pattern
     return bool(_REF.match(s))
 
 @_repeat
 def _repair_refs(x):
-    """Performs the repair on the element list 'x'."""
+    """Performs the repair on the element list `x`."""
 
     if not bool(_PANDOCVERSION):
         raise RuntimeError('Module uninitialized.  Please call init().')
@@ -511,7 +607,7 @@ def _repair_refs(x):
                           x[i+1]['t'], x[i+1]['c'] if 'c' in x[i+1] else []):
 
             # Get the reference string
-            n = 0 if _PANDOCVERSION < '1.16' else 1
+            n = 0 if version(_PANDOCVERSION) < version('1.16') else 1
             s = x[i]['c'][n][0]['c'] + x[i+1]['c']
 
             # Chop it into pieces.  Note that the prefix and suffix may be
@@ -544,12 +640,12 @@ def _repair_refs(x):
     return True  # Terminates processing
 
 def repair_refs(key, value, fmt, meta):  # pylint: disable=unused-argument
-    """Using "-f markdown+autolink_bare_uris" with pandoc < 1.18 splits a
-    reference like "{@fig:one}" into email Link and Str elements.  This
+    """Using '-f markdown+autolink_bare_uris' with pandoc < 1.18 splits a
+    reference like '{@fig:one}' into email Link and Str elements.  This
     function replaces the mess with the Cite and Str elements we normally
     get.  Call this before any reference processing."""
 
-    if _PANDOCVERSION >= '1.18':
+    if version(_PANDOCVERSION) >= version('1.18'):
         return
 
     # The problem spans multiple elements, and so can only be identified in
@@ -567,9 +663,9 @@ def repair_refs(key, value, fmt, meta):  # pylint: disable=unused-argument
 # process_refs_factory() -----------------------------------------------------
 
 def _extract_modifier(x, i, attrs):
-    """Extracts the */+/! modifier in front of the Cite at index 'i' of the
-    element list 'x'.  The modifier is stored in 'attrs'.
-    Returns the updated index 'i'.
+    """Extracts the */+/! modifier in front of the Cite at index `i` of the
+    element list `x`.  The modifier is stored in `attrs`.  Returns the
+    updated index `i`.
     """
 
     global _cleveref_flag  # pylint: disable=global-statement
@@ -609,14 +705,13 @@ def _extract_modifier(x, i, attrs):
             else:
                 del x[i-1]
                 i -= 1
-
     return i
 
 def _remove_brackets(x, i):
-    """Removes curly brackets surrounding the Cite element at index 'i' in
-    the element list 'x'.  It is assumed that the modifier has been
-    extracted.  Empty strings are deleted from 'x'.
-    Returns the updated index 'i'.
+    """Removes curly brackets surrounding the Cite element at index `i` in
+    the element list `x`.  It is assumed that the modifier has been
+    extracted.  Empty strings are deleted from `x`.  Returns the updated
+    index `i`.
     """
 
     assert x[i]['t'] == 'Cite'
@@ -660,12 +755,12 @@ def _remove_brackets(x, i):
 badlabels = []
 
 @_repeat
-def _process_refs(name, x, patt, labels, warninglevel):
-    """Strips surrounding curly braces and adds modifiers to the
-    attributes of Cite elements.  Only references with labels that match the
-    pattern `patt` or are in the `labels` list are processed.  Repeats
-    processing (via decorator) until no more unprocessed references are
-    found."""
+def _process_refs(x, pattern, labels):
+    """Searches the element list `x` for the first Cite element with an id
+    that either matches the compiled regular expression `pattern` or is found in
+    the `labels` list.  Strips surrounding curly braces and adds modifiers to
+    the attributes of the Cite element.  Repeats processing (via decorator)
+    until all matching Cite elements in `x` are processed."""
 
     # Scan the element list x for Cite elements with known labels
     for i, v in enumerate(x):
@@ -677,7 +772,7 @@ def _process_refs(name, x, patt, labels, warninglevel):
                 if testlabel in labels:
                     label = testlabel
 
-            if (patt and patt.match(label)) or label in labels:
+            if (pattern and pattern.match(label)) or label in labels:
 
                 # A new reference was found; create some empty attrs for it
                 attrs = PandocAttributes()
@@ -709,71 +804,98 @@ def _process_refs(name, x, patt, labels, warninglevel):
                 if label in labels:
                     return None  # Forces processing to repeat via @_repeat
 
-            if warninglevel and patt and \
-              patt.match(label) and label not in badlabels:
+            if _WARNINGLEVEL and pattern and \
+              pattern.match(label) and label not in badlabels:
                 badlabels.append(label)
-                msg = "\n%s: Bad reference: @%s.\n" % (name, label)
+                msg = "\n%s: Bad reference: @%s.\n" % (_FILTERNAME, label)
                 STDERR.write(msg)
+                STDERR.flush()
 
     return True  # Terminates processing in _repeat decorator
 
-def process_refs_factory(name, patt, labels, warninglevel):
+@_compat
+def process_refs_factory(regex, labels, warninglevel=None):
     """Returns process_refs(key, value, fmt, meta) action that processes
-    text around a reference.  Only references with labels found in the
-    'labels' list are processed.
+    text around a reference.  References are encapsulated in pandoc Cite
+    elements.
 
-    Consider the markdown "{+@fig:1}", which represents a reference to a
-    figure. "@" denotes a reference, "fig:1" is the reference's label, and
-    "+" is a modifier.  Valid modifiers are '+, '*' and '!'.
+    Consider the markdown '{+@fig:1}', which represents a reference to a
+    figure. '@' denotes a reference, 'fig:1' is the reference's label, and
+    '+' is a modifier.  Valid modifiers are '+', '*' and '!'.
 
-    This function strips curly braces and adds the modifiers to the attributes
-    of Cite elements.  Cite attributes must be detached before the document is
-    written to STDOUT because pandoc doesn't recognize them.  Alternatively,
-    use an action from replace_refs_factory() to replace the references
+    Only references with labels that match the regular expression `regex` or
+    are found in the `labels` list are processed.  Curly braces are stripped
+    and modifiers are stored in the `modifier` field of the Cite element's
+    attributes.
+
+    Cite attributes must be detached before the document is written to
+    STDOUT because pandoc doesn't recognize them.  Alternatively, an action
+    from replace_refs_factory() can be used to replace the references
     altogether.
+
+    Parameters:
+
+      regex - regular expression (or compiled pattern) that matches references
+      labels - a list of known target labels
+      warninglevel - DEPRECATED (set global WARNINGLEVEL instead)
     """
 
+    # Set the global warning level (DEPRECATED)
+    # pylint: disable=global-statement
+    global _WARNINGLEVEL
+    assert(warninglevel is None or isinstance(warninglevel, int))
+    if warninglevel is not None:
+        _WARNINGLEVEL = warninglevel
+
+    # Compile the regex if it is a string; otherwise assume it is either a
+    # compiled pattern or None.
+    pattern = re.compile(regex) if isinstance(regex, STRTYPES) else regex
+
     # pylint: disable=unused-argument
     def process_refs(key, value, fmt, meta):
         """Processes references."""
         # References may occur in a variety of places; we must process them
         # all.
 
-        if key in ['Para', 'Plain']:
-            _process_refs(name, value, patt, labels, warninglevel)
+        if key in ['Para', 'Plain', 'Emph', 'Strong']:
+            _process_refs(value, pattern, labels)
+        elif key in ['Span', 'Header']:
+            _process_refs(value[-1], pattern, labels)
         elif key == 'Image':
-            _process_refs(name, value[-2], patt, labels, warninglevel)
+            _process_refs(value[-2], pattern, labels)
         elif key == 'Table':
-            _process_refs(name, value[-5], patt, labels, warninglevel)
-        elif key == 'Span':
-            _process_refs(name, value[-1], patt, labels, warninglevel)
-        elif key == 'Emph':
-            _process_refs(name, value, patt, labels, warninglevel)
-        elif key == 'Strong':
-            _process_refs(name, value, patt, labels, warninglevel)
+            if version(_PANDOCVERSION) < version('2.10'):
+                _process_refs(value[-5], pattern, labels)
+            else:
+                if value[-5]['c'][1]:
+                    _process_refs(value[-5]['c'][1][0]['c'], pattern, labels)
         elif key == 'Cite':
-            _process_refs(name, value[-2][0]['citationPrefix'], patt, labels,
-                          warninglevel)
-            _process_refs(name, value[-2][0]['citationSuffix'], patt, labels,
-                          warninglevel)
+            _process_refs(value[-2][0]['citationPrefix'], pattern, labels)
+            _process_refs(value[-2][0]['citationSuffix'], pattern, labels)
 
     return process_refs
 
 
 # replace_refs_factory() ------------------------------------------------------
 
+# Type for target metadata
+Target = collections.namedtuple('Target', ['num', 'secno', 'has_duplicate'])
+Target.__new__.__defaults__ = (None,) * len(Target._fields)
+
 # pylint: disable=too-many-arguments,unused-argument
 def replace_refs_factory(references, use_cleveref_default, use_eqref,
                          plusname, starname, allow_implicit_refs=False):
     """Returns replace_refs(key, value, fmt, meta) action that replaces
-    references with format-specific content.  The content is determined using
-    the 'references' dict, which maps each reference label to a
-    [number/tag, secno] list (e.g.,
-    { 'fig:1':[1, '1'], 'fig:2':[2,'1'], ...}).  If 'use_cleveref_default'
-    is True, or if "modifier" in the reference's attributes is "+" or "*", then
-    clever referencing is used; i.e., a name is placed in front of the number
-    or string tag.  The 'plusname' and 'starname' lists give the singular
-    and plural names for "+" and "*" clever references, respectively."""
+    references encapsulated in Cite elements with format-specific content.
+    The content is determined using the `references` dict, which maps
+    reference labels to Target objects.
+
+    If `use_cleveref_default` is True, or if `modifier` in the reference's
+    attributes is '+' or '*, then clever referencing is used; i.e., a name is
+    placed in front of the number or string tag.  The`'plusname` and `starname`
+    lists give the singular and plural names for '+' and '*' clever references,
+    respectively.
+    """
 
     global _cleveref_flag  # pylint: disable=global-statement
 
@@ -786,31 +908,47 @@ def replace_refs_factory(references, use_cleveref_default, use_eqref,
 
         assert key == 'Cite'
 
+        # Extract the attributes
         attrs = PandocAttributes(value[0], 'pandoc')
 
+        # Check if the nolink attribute is set
         nolink = attrs['nolink'].capitalize() == 'True' if 'nolink' in attrs \
           else False
 
+        # Extract the label
         label = value[-2][0]['citationId']
         if allow_implicit_refs and not label in references and ':' in label:
             testlabel = label.split(':')[-1]
             if testlabel in references:
                 label = testlabel
 
+        # Get the target metadata; typecast it as a Target for easier access
+        target = references[label] if label in references else None
+        if target and not isinstance(target, Target):
+            target = Target(*target)
+
+        # Issue a warning for duplicate targets
+        if _WARNINGLEVEL and target and target.has_duplicate:
+            msg = textwrap.dedent("""
+                %s: Referenced label has duplicate: %s
+            """ % (_FILTERNAME, label))
+            STDERR.write(msg)
+            STDERR.flush()
+
         # Get the replacement value
-        text = str(references[label][0]) if label in references else '??'
+        text = str(target.num) if target else '??'
 
         # Choose between \Cref, \cref and \ref
         use_cleveref = attrs['modifier'] in ['*', '+'] \
           if 'modifier' in attrs else use_cleveref_default
-        plus = attrs['modifier'] == '+' if 'modifier' in attrs \
+        is_plus_ref = attrs['modifier'] == '+' if 'modifier' in attrs \
           else use_cleveref_default
-        name = plusname[0] if plus else starname[0]  # Name used by cref
+        refname = plusname[0] if is_plus_ref else starname[0]  # Reference name
 
-        # The replacement depends on the output format
+        # The replacement content depends on the output format
         if fmt == 'latex':
             if use_cleveref:
-                macro = r'\cref' if plus else r'\Cref'
+                macro = r'\cref' if is_plus_ref else r'\Cref'
                 ret = RawInline('tex', r'%s{%s}'%(macro, label))
             elif use_eqref:
                 ret = RawInline('tex', r'\eqref{%s}'%label)
@@ -827,17 +965,17 @@ def replace_refs_factory(references, use_cleveref_default, use_eqref,
               if text.startswith('$') and text.endswith('$') \
               else Str(text)
 
-            if not nolink and label in references:
-                prefix = 'ch%03d.xhtml' % references[label][1] \
+            if not nolink and target:
+                prefix = 'ch%03d.xhtml' % target.secno \
                   if fmt in ['epub', 'epub2', 'epub3'] and \
-                  references[label][1] else ''
+                  target.secno else ''
 
                 elem = elt('Link', 2)([elem],
                                       ['%s#%s' % (prefix, label), '']) \
-                  if _PANDOCVERSION < '1.16' else \
+                  if version(_PANDOCVERSION) < version('1.16') else \
                   Link(['', [], []], [elem], ['%s#%s' % (prefix, label), ''])
 
-            ret = ([Str(name), Space()] if use_cleveref else []) + [elem]
+            ret = ([Str(refname + ' ')] if use_cleveref else []) + [elem]
 
         # If the Cite was square-bracketed then wrap everything in a span
         s = stringify(value[-1])
@@ -871,16 +1009,35 @@ def replace_refs_factory(references, use_cleveref_default, use_eqref,
 # attach_attrs_factory() -----------------------------------------------------
 
 # pylint: disable=redefined-outer-name
-def attach_attrs_factory(name, f, warninglevel, extract_attrs=extract_attrs,
+@_compat
+def attach_attrs_factory(f, warninglevel=None,
+                         extract_attrs=extract_attrs,
                          allow_space=False, replace=False):
     """Returns attach_attrs(key, value, fmt, meta) action that reads and
     attaches attributes to unattributed elements generated by the
-    pandocfilters function f (e.g. pandocfilters.Math, etc).
+    pandocfilters function `f` (e.g. pandocfilters.Math, etc).
 
     The extract_attrs() function should read and return the attributes and
     raise a ValueError or IndexError if attributes are not found.
+
+    Parameters:
+
+      f - the pandoc constructor for the elements of interest
+      warninglevel - DEPRECATED (set global WARNINGLEVEL instead)
+      extract_attrs - a function to extract attributes from an element list;
+                      defaults to the extract_attrs() function in this module
+      allow_space - flags that a space should be allowed between an element and
+                    its attributes
+      replace - flags that existing attributes should be replaced
     """
 
+    # Set the global warning level (DEPRECATED)
+    # pylint: disable=global-statement
+    global _WARNINGLEVEL
+    assert(warninglevel is None or isinstance(warninglevel, int))
+    if warninglevel is not None:
+        _WARNINGLEVEL = warninglevel
+
     # Get the name of the element from the function
     elname = f.__closure__[0].cell_contents
 
@@ -894,14 +1051,13 @@ def attach_attrs_factory(name, f, warninglevel, extract_attrs=extract_attrs,
                     n += 1
                 try:  # Extract the attributes
                     attrs = extract_attrs(x, n)
-                    if attrs.parse_failed and warninglevel:
-                        msg = textwrap.dedent("""\
+                    if _WARNINGLEVEL and attrs.parse_failed:
+                        msg = textwrap.dedent("""
                             %s: Malformed attributes:
                             %s
-                        """ % (name, attrs.attrstr))
-                        STDERR.write('\n')
+                        """ % (_FILTERNAME, attrs.attrstr))
                         STDERR.write(msg)
-                        STDERR.write('\n')
+                        STDERR.flush()
 
                     if replace:
                         x[i]['c'][0] = attrs.list
@@ -937,8 +1093,8 @@ def attach_attrs_factory(name, f, warninglevel, extract_attrs=extract_attrs,
 
 def detach_attrs_factory(f, restore=False):
     """Returns detach_attrs(key, value, fmt, meta) action that detaches
-    attributes attached to elements of type f (e.g. pandocfilters.Math, etc).
-    Attributes provided natively by pandoc will be left as is."""
+    attributes attached to elements of type `f` (e.g. pandocfilters.Math, etc).
+    Attributes provided natively by pandoc are left as is."""
 
     # Get the name and standard length
     name = f.__closure__[0].cell_contents
@@ -965,10 +1121,9 @@ def detach_attrs_factory(f, restore=False):
 
 # insert_secnos_factory() ----------------------------------------------------
 
-# pylint: disable=redefined-outer-name
 def insert_secnos_factory(f):
     """Returns insert_secnos(key, value, fmt, meta) action that inserts
-    section numbers into the attributes of elements of type f.
+    section numbers into the attributes of elements of type `f`.
     """
 
     # Get the name and standard length
@@ -988,10 +1143,16 @@ def insert_secnos_factory(f):
         if key == name:
 
             # Only insert if attributes are attached.  Images always have
-            # attributes for pandoc >= 1.16.
+            # attributes for pandoc >= 1.16. Same for Spans.  Tables always
+            # have attributes for pandoc >= 10.1.
             assert len(value) <= n+1
+            # pylint: disable=too-many-boolean-expressions
             if (name == 'Image' and len(value) == 3) or \
+              name == 'Div' or \
+              name == 'Span' or \
+              (name == 'Table' and len(value) == 6) or \
               len(value) == n+1:
+
                 # Make sure value[0] represents attributes
                 assert isinstance(value[0][0], STRTYPES)
                 assert isinstance(value[0][1], list)
@@ -1005,10 +1166,9 @@ def insert_secnos_factory(f):
 
 # delete_secnos_factory() ----------------------------------------------------
 
-# pylint: disable=redefined-outer-name
 def delete_secnos_factory(f):
     """Returns delete_secnos(key, value, fmt, meta) action that deletes
-    section numbers from the attributes of elements of type f.
+    section numbers from the attributes of elements of type `f`.
     """
 
     # Get the name and standard length
@@ -1019,10 +1179,16 @@ def delete_secnos_factory(f):
         """Deletes section numbers from elements attributes."""
 
         # Only delete if attributes are attached.   Images always have
-        # attributes for pandoc >= 1.16.
+        # attributes for pandoc >= 1.16. Same for Spans.    Tables always
+            # have attributes for pandoc >= 10.1.
         if key == name:
             assert len(value) <= n+1
-            if (name == 'Image' and len(value) == 3) or len(value) == n+1:
+            # pylint: disable=too-many-boolean-expressions
+            if (name == 'Image' and len(value) == 3) or \
+              name == 'Div' or \
+              name == 'Span' or \
+              (name == 'Table' and len(value) == 6) or \
+              len(value) == n+1:
 
                 # Make sure value[0] represents attributes
                 assert isinstance(value[0][0], STRTYPES)
