@@ -2,7 +2,7 @@
 
 """Unit tests for pandoc-xnos."""
 
-# Copyright 2016-2019 Thomas J. Duck.
+# Copyright 2016-2020 Thomas J. Duck.
 # All rights reserved.
 #
 # This program is free software: you can redistribute it and/or modify
@@ -35,10 +35,10 @@ from pandocxnos import attach_attrs_factory, detach_attrs_factory
 from pandocxnos import insert_secnos_factory
 from pandocxnos import repair_refs, process_refs_factory, replace_refs_factory
 
-PANDOCVERSION = '2.7'
+PANDOCVERSION = '2.8.1'
+PANDOC = 'pandoc-2.10.1'
 PANDOC1p15 = 'pandoc-1.15.2'  # pylint: disable=invalid-name
-
-PANDOC_API_VERSION = '1,17,5,4'
+PANDOC_API_VERSION = '1,21'
 
 
 #-----------------------------------------------------------------------------
@@ -63,7 +63,7 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', ''), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json -M foo=bar'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json -M foo=bar').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         expected = 'bar'
@@ -84,7 +84,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', '---\nfoo: bar\n...'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         expected = 'bar'
@@ -105,7 +105,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', '---\nfoo:\n  - bar\n  - baz\n...'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         expected = ['bar', 'baz']
@@ -120,13 +120,13 @@ class TestXnos(unittest.TestCase):
         ## test.md: ---\nfoo: True\n... ##
 
         # Command: pandoc test.md -t json
-        src = eval(r'''{"blocks": [], "pandoc-api-version": [1, 17, 5, 4], "meta": {"foo": {"t": "MetaBool", "c": True}}}''')
+        src = eval(r'''{"blocks": [], "pandoc-api-version": [%s], "meta": {"foo": {"t": "MetaBool", "c": True}}}'''%PANDOC_API_VERSION)
 
         # Check src against current pandoc
         md = subprocess.Popen(('echo', '---\nfoo: True\n...'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(),
+            (PANDOC+' -t json').split(),
             stdin=md.stdout).strip().decode("utf-8").replace('true', 'True'))
         self.assertEqual(src, output)
 
@@ -142,23 +142,22 @@ class TestXnos(unittest.TestCase):
         ## test.md: ---\nxnos-number-sections: True\n...\n\n# Title\n\n$$ x $$ {#eq:1}\n ##
 
         # Command: pandoc test.md -t json
-        src = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [{"t": "DisplayMath"}, " x "]}, {"t": "Space"}, {"t": "Str", "c": "{#eq:1}"}]}], "pandoc-api-version": [1, 17, 5, 4], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}''')
+        src = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [{"t": "DisplayMath"}, " x "]}, {"t": "Space"}, {"t": "Str", "c": "{#eq:1}"}]}], "pandoc-api-version": [%s], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}'''%PANDOC_API_VERSION)
 
         # Check src against current pandoc
         md = subprocess.Popen(('echo', '---\nxnos-number-sections: True\n...\n\n# Title\n\n$$ x $$ {#eq:1}\n'), stdout=subprocess.PIPE)
 
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(),
+            (PANDOC+' -t json').split(),
             stdin=md.stdout).strip().decode("utf-8").replace('true', 'True'))
         self.assertEqual(src, output)
 
-        expected = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [["eq:1", [], [["secno", 1]]], {"t": "DisplayMath"}, " x "]}, {"t": "Space"}]}], "pandoc-api-version": [1, 17, 5, 4], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}''')
+        expected = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [["eq:1", [], [["secno", 1]]], {"t": "DisplayMath"}, " x "]}, {"t": "Space"}]}], "pandoc-api-version": [%s], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}'''%PANDOC_API_VERSION)
 
         # Make the comparison
         meta = src['meta']
         fmt = 'html'
-        attach_attrs_math = attach_attrs_factory(None, Math, 0,
-                                                 allow_space=True)
+        attach_attrs_math = attach_attrs_factory(Math, 0, allow_space=True)
         insert_secnos = insert_secnos_factory(Math)
         tmp = walk(src, attach_attrs_math, fmt, meta)
         self.assertEqual(walk(tmp, insert_secnos, fmt, meta), expected)
@@ -170,23 +169,22 @@ class TestXnos(unittest.TestCase):
         ## test.md: ---\nxnos-number-sections: True\n...\n\n# Title\n\n$$ x $$\n ##
 
         # Command: pandoc test.md -t json
-        src = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [{"t": "DisplayMath"}, " x "]}]}], "pandoc-api-version": [1, 17, 5, 4], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}''')
+        src = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [{"t": "DisplayMath"}, " x "]}]}], "pandoc-api-version": [%s], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}'''%PANDOC_API_VERSION)
 
         # Check src against current pandoc
         md = subprocess.Popen(('echo', '---\nxnos-number-sections: True\n...\n\n# Title\n\n$$ x $$\n'), stdout=subprocess.PIPE)
 
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(),
+            (PANDOC+' -t json').split(),
             stdin=md.stdout).strip().decode("utf-8").replace('true', 'True'))
         self.assertEqual(src, output)
 
-        expected = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [{"t": "DisplayMath"}, " x "]}]}], "pandoc-api-version": [1, 17, 5, 4], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}''')
+        expected = eval(r'''{"blocks": [{"t": "Header", "c": [1, ["title", [], []], [{"t": "Str", "c": "Title"}]]}, {"t": "Para", "c": [{"t": "Math", "c": [{"t": "DisplayMath"}, " x "]}]}], "pandoc-api-version": [%s], "meta": {"xnos-number-sections": {"t": "MetaBool", "c": True}}}'''%PANDOC_API_VERSION)
 
         # Make the comparison
         meta = src['meta']
         fmt = 'html'
-        attach_attrs_math = attach_attrs_factory(None, Math, 0,
-                                                 allow_space=True)
+        attach_attrs_math = attach_attrs_factory(Math, 0, allow_space=True)
         insert_secnos = insert_secnos_factory(Math)
         tmp = walk(src, attach_attrs_math, fmt, meta)
         self.assertEqual(walk(tmp, insert_secnos, fmt, meta), expected)
@@ -213,7 +211,7 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', '"test"'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -f markdown+smart -t json'.split(),
+            (PANDOC+' -f markdown+smart -t json').split(),
             stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
@@ -223,7 +221,7 @@ class TestXnos(unittest.TestCase):
         # Check expected against current pandoc
         md = subprocess.Popen(('echo', '"test"'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -f markdown-smart -t json'.split(),
+            (PANDOC+' -f markdown-smart -t json').split(),
             stdin=md.stdout).strip())
         self.assertEqual(expected, output)
 
@@ -243,7 +241,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', "This is 'test 2'."),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -f markdown+smart -t json'.split(),
+            (PANDOC+' -f markdown+smart -t json').split(),
             stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
@@ -254,7 +252,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', "This is 'test 2'."),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -f markdown-smart -t json'.split(),
+            (PANDOC+' -f markdown-smart -t json').split(),
             stdin=md.stdout).strip())
         self.assertEqual(expected, output)
 
@@ -274,7 +272,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', r'$\frac{1}{2}$'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded
@@ -296,7 +294,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', 'Test {#eq:id .class tag="foo"}.'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded
@@ -318,7 +316,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', 'Test {#eq:id .class tag="foo"}.'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -f markdown+smart -t json'.split(),\
+            (PANDOC+' -f markdown+smart -t json').split(),\
               stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
@@ -476,7 +474,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', 'As shown in @fig:one.'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(),
+            (PANDOC+' -t json').split(),
             stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
@@ -484,7 +482,7 @@ class TestXnos(unittest.TestCase):
         expected = eval(r'''{"meta":{},"blocks":[{"t":"Para","c":[{"t":"Str","c":"As"},{"t":"Space"},{"t":"Str","c":"shown"},{"t":"Space"},{"t":"Str","c":"in"},{"t":"Space"},{"t":"Cite","c":[["",[],[]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"fig:one","citationHash":0}],[{"t":"Str","c":"@fig:one"}]]},{"t":"Str","c":"."}]}],"pandoc-api-version":[%s]}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['fig:one'], 0)
+        process_refs = process_refs_factory(None, ['fig:one'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
 
@@ -500,7 +498,7 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', '(@eq:one)'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(),
+            (PANDOC+' -t json').split(),
             stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
@@ -508,7 +506,7 @@ class TestXnos(unittest.TestCase):
         expected = eval(r'''{"blocks":[{"t":"Para","c":[{"t":"Str","c":"("},{"t":"Cite","c":[["",[],[]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"eq:one","citationHash":0}],[{"t":"Str","c":"@eq:one"}]]},{"t":"Str","c":")"}]}],"pandoc-api-version":[%s],"meta":{}}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['eq:one'], 0)
+        process_refs = process_refs_factory(None, ['eq:one'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
 
@@ -523,14 +521,14 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', 'See {@tbl:1}.'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded (braces stripped, attributes added)
         expected = eval(r'''{"meta":{},"blocks":[{"t":"Para","c":[{"t":"Str","c":"See"},{"t":"Space"},{"t":"Cite","c":[["",[],[]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"tbl:1","citationHash":0}],[{"t":"Str","c":"@tbl:1"}]]},{"t":"Str","c":"."}]}],"pandoc-api-version":[%s],"meta":{}}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['tbl:1'], 0)
+        process_refs = process_refs_factory(None, ['tbl:1'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
 
@@ -545,14 +543,14 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', 'See +@eq:1.'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded (modifier extracted, attributes added)
         expected = eval(r'''{"meta":{},"blocks":[{"t":"Para","c":[{"t":"Str","c":"See"},{"t":"Space"},{"t":"Cite","c":[["",[],[["modifier","+"]]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"eq:1","citationHash":0}],[{"t":"Str","c":"@eq:1"}]]},{"t":"Str","c":"."}]}],"pandoc-api-version":[%s]}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['eq:1'], 0)
+        process_refs = process_refs_factory(None, ['eq:1'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
 
@@ -567,14 +565,14 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', 'See {+@tbl:1}.'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded (braces stripped, modifier extracted, attributes added)
         expected = eval(r'''{"meta":{},"blocks":[{"t":"Para","c":[{"t":"Str","c":"See"},{"t":"Space"},{"t":"Cite","c":[["",[],[["modifier","+"]]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"tbl:1","citationHash":0}],[{"t":"Str","c":"@tbl:1"}]]},{"t":"Str","c":"."}]}],"pandoc-api-version":[%s]}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['tbl:1'], 0)
+        process_refs = process_refs_factory(None, ['tbl:1'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
 
@@ -589,14 +587,14 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', 'See xxx{+@tbl:1}xxx.'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded (braces stripped, modifier extracted, attributes added)
         expected = eval(r'''{"meta":{},"blocks":[{"t":"Para","c":[{"t":"Str","c":"See"},{"t":"Space"},{"t":"Str","c":"xxx"},{"t":"Cite","c":[["",[],[["modifier","+"]]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"tbl:1","citationHash":0}],[{"t":"Str","c":"@tbl:1"}]]},{"t":"Str","c":"xxx."}]}],"pandoc-api-version":[%s]}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['tbl:1'], 0)
+        process_refs = process_refs_factory(None, ['tbl:1'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
 
@@ -611,14 +609,14 @@ class TestXnos(unittest.TestCase):
         # Check src against current pandoc
         md = subprocess.Popen(('echo', 'See [+@eq:1].'), stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded (modifier extracted, attributes added)
         expected = eval(r'''{"meta":{},"blocks":[{"t":"Para","c":[{"t":"Str","c":"See"},{"t":"Space"},{"t":"Cite","c":[["",[],[["modifier","+"]]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"NormalCitation"},"citationPrefix":[],"citationId":"eq:1","citationHash":0}],[{"t":"Str","c":"[+@eq:1]"}]]},{"t":"Str","c":"."}]}],"pandoc-api-version":[%s]}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['eq:1'], 0)
+        process_refs = process_refs_factory(None, ['eq:1'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
 
@@ -635,15 +633,14 @@ class TestXnos(unittest.TestCase):
             ('echo', '{+@tbl:one}-{@tbl:four} provide the data.'),
             stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded
         expected = eval(r'''{"meta":{},"blocks":[{"t":"Para","c":[{"t":"Cite","c":[["",[],[["modifier","+"]]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"tbl:one","citationHash":0}],[{"t":"Str","c":"@tbl:one"}]]},{"t":"Str","c":"-"},{"t":"Cite","c":[["",[],[]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText"},"citationPrefix":[],"citationId":"tbl:four","citationHash":0}],[{"t":"Str","c":"@tbl:four"}]]},{"t":"Space"},{"t":"Str","c":"provide"},{"t":"Space"},{"t":"Str","c":"the"},{"t":"Space"},{"t":"Str","c":"data."}]}],"pandoc-api-version":[%s]}'''%PANDOC_API_VERSION)
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None,
-                                            ['tbl:one', 'tbl:four'], 0)
+        process_refs = process_refs_factory(None, ['tbl:one', 'tbl:four'], 0)
         self.assertEqual(walk(src, process_refs, '', {}), expected)
 
     @unittest.skip('Known issue for pandoc-1.15.2')
@@ -670,10 +667,10 @@ class TestXnos(unittest.TestCase):
         # Generate expected using current pandoc
         md = subprocess.Popen(('echo', '@fig:1:'), stdout=subprocess.PIPE)
         expected = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['fig:1'], 0)
+        process_refs = process_refs_factory(None, ['fig:1'], 0)
         self.assertEqual(walk(src, process_refs, {}, ''), expected)
 
 
@@ -697,7 +694,7 @@ class TestXnos(unittest.TestCase):
         expected = eval(r'''[{"unMeta":{}},[{"t":"Para","c":[{"t":"Cite","c":[["",[],[]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText","c":[]},"citationPrefix":[],"citationId":"fig:1","citationHash":0}],[{"t":"Str","c":"@fig:1"}]]},{"t":"Str","c":":"}]}]]''')
 
         # Make the comparison
-        process_refs = process_refs_factory(None, None, ['fig:1'], 0)
+        process_refs = process_refs_factory(None, ['fig:1'], 0)
         self.assertEqual(walk(src, process_refs, {}, ''), expected)
 
 
@@ -710,10 +707,11 @@ class TestXnos(unittest.TestCase):
         src = eval(r'''[{"unMeta":{}},[{"t":"Para","c":[{"t":"Str","c":"As"},{"t":"Space","c":[]},{"t":"Str","c":"shown"},{"t":"Space","c":[]},{"t":"Str","c":"in"},{"t":"Space","c":[]},{"t":"Cite","c":[["",[],[]],[{"citationSuffix":[],"citationNoteNum":0,"citationMode":{"t":"AuthorInText","c":[]},"citationPrefix":[],"citationId":"fig:one","citationHash":0}],[{"t":"Str","c":"@fig:one"}]]},{"t":"Str","c":"."}]}]]''')
 
         # Hand-coded
-        expected = eval(r'''[{"unMeta":{}},[{"t":"Para","c":[{"t":"Str","c":"As"},{"t":"Space","c":[]},{"t":"Str","c":"shown"},{"t":"Space","c":[]},{"t":"Str","c":"in"},{"t":"Space","c":[]},{"t":"Str","c":"fig."},{"t":"Space","c":[]},{'t':'Link','c':[['',[],[]],[{'t':'Str','c':'1'}],['#fig:one','']]},{"t":"Str","c":"."}]}]]''')
+        expected = eval(r'''[{"unMeta":{}},[{"t":"Para","c":[{"t":"Str","c":"As"},{"t":"Space","c":[]},{"t":"Str","c":"shown"},{"t":"Space","c":[]},{"t":"Str","c":"in"},{"t":"Space","c":[]},{"t":"Str","c":"fig. "},{'t':'Link','c':[['',[],[]],[{'t':'Str','c':'1'}],['#fig:one','']]},{"t":"Str","c":"."}]}]]''')
 
         # Make the comparison
-        replace_refs = replace_refs_factory({'fig:one':[1, 1]}, True, False,
+        replace_refs = replace_refs_factory({'fig:one':pandocxnos.Target(1, 1)},
+                                            True, False,
                                             ['fig.', 'figs.'],
                                             ['Figure', 'Figures'])
         self.assertEqual(walk(walk(src, replace_refs, {}, ''),
@@ -723,8 +721,7 @@ class TestXnos(unittest.TestCase):
     def test_attach_attrs_factory(self):
         """Tests attach_attrs_math()."""
 
-        attach_attrs_math = attach_attrs_factory(None, Math, 0,
-                                                 allow_space=True)
+        attach_attrs_math = attach_attrs_factory(Math, 0, allow_space=True)
 
         ## test.md: $$ y = f(x) $${#eq:1 tag="B.1"} ##
 
@@ -735,7 +732,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', '$$ y = f(x) $${#eq:1 tag="B.1"}'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(src, output)
 
         # Hand-coded (attributes deleted)
@@ -762,7 +759,7 @@ class TestXnos(unittest.TestCase):
         md = subprocess.Popen(('echo', '$$ y = f(x) $$'),
                               stdout=subprocess.PIPE)
         output = eval(subprocess.check_output(
-            'pandoc -t json'.split(), stdin=md.stdout).strip())
+            (PANDOC+' -t json').split(), stdin=md.stdout).strip())
         self.assertEqual(expected, output)
 
         # Make the comparison
